@@ -5,7 +5,7 @@
 
     Flask-CORS extension module
 """
-
+from _version import __version__
 import collections
 
 from datetime import timedelta
@@ -19,13 +19,14 @@ AccessControlAllowOrigin = 'Access-Control-Allow-Origin'
 
 def cross_origin(origins=None, methods=None, headers=None,
                  supports_credentials=False, max_age=None, send_wildcard=True,
-                 always_send=True, automatic_options=False):
+                 always_send=True, automatic_options=True):
     '''
-    This function is the decorator which is used to wrap a Flask route with. In
-    the simplest case, simply use the default parameters to allow all origins
-    in what is the most permissive configuration. If this method modifies state
-    or performs authentication which may be brute-forced, you should add some
-    degree of protection, for example Cross Site Forgery Request protection.
+    This function is the decorator which is used to wrap a Flask route with.
+    In the simplest case, simply use the default parameters to allow all
+    origins in what is the most permissive configuration. If this method
+    modifies state or performs authentication which may be brute-forced, you
+    should add some degree of protection, for example Cross Site Forgery
+    Request protection.
 
 
     :param origins: The origin, or list of origins which are to be allowed,
@@ -46,21 +47,24 @@ def cross_origin(origins=None, methods=None, headers=None,
          Note: this option cannot be used in conjuction with a '*' origin
     :type supports_credentials: bool
 
-    :param max_age: The maximum time for which this CORS request may be cached.
-        This value is set as the `Access-Control-Max-Age` header.
+    :param max_age: The maximum time for which this CORS request maybe cached.
+                    This value is set as the `Access-Control-Max-Age` header.
     :type max_age: timedelta, integer, string or None
 
-    :param send_wildcard: If True, and the origins parameter is `*`, a wildcard
-                          `Access-Control-Allow-Origin` header is sent, rather
-                          than echoing the request's `Origin` header.
+    :param send_wildcard: If True, and the origins parameter is `*`, a
+                          wildcard `Access-Control-Allow-Origin` header is
+                          sent, rather than echoing the request's `Origin`
+                          header.
     :type send_wildcard: bool
 
     :param always_send: If True, CORS headers are sent even if there is no
-        `Origin` in the request's headers.
+                        `Origin` in the request's headers.
     :type always_send: bool
 
-    :param automatic_options: If True, Flask's automatic_options is enabled,
-        otherwise the default Flask-Cors behavior is used.
+    :param automatic_options: If True, CORS headers will be returned for
+        OPTIONS requests. For use with cross domain POST requests which
+        preflight OPTIONS requests, you will need to specifically allow
+        the Content-Type header.
     :type automatic_options: bool
 
     '''
@@ -68,7 +72,8 @@ def cross_origin(origins=None, methods=None, headers=None,
     methods = methods or ['GET', 'HEAD', 'POST', 'OPTIONS', 'PUT']
     methods = ', '.join(sorted(x for x in methods)).upper()
 
-    if not isinstance(headers, string_types) and isinstance(headers, collections.Iterable):
+    if (not isinstance(headers, string_types)
+            and isinstance(headers, collections.Iterable)):
         headers = ', '.join(x for x in headers)
 
     wildcard = origins == '*'
@@ -82,20 +87,22 @@ def cross_origin(origins=None, methods=None, headers=None,
             origins = _origins or current_app.config.get('CORS_ORIGINS', '*')
             origins_str = str(origins)
 
-            if not isinstance(origins, string_types) and isinstance(origins, collections.Iterable):
+            if(not isinstance(origins, string_types)
+                    and isinstance(origins, collections.Iterable)):
                 origins_str = ', '.join(origins)
 
             # If the Origin header is not present terminate this set of steps.
             # The request is outside the scope of this specification.
             request_origin = request.headers.get('Origin', '')
 
-            if not 'Origin' in request.headers and not always_send:
+            if 'Origin' not in request.headers and not always_send:
                 return make_response(f(*args, **kwargs))
 
             # If the value of the Origin header is not a case-sensitive match
             # for any of the values in list of origins, do not set any
             # additional headers and terminate this set of steps.
-            elif not wildcard and not always_send and not request_origin in origins:
+            elif(not wildcard and not always_send and
+                    request_origin not in origins):
                 return make_response(f(*args, **kwargs))
 
             if automatic_options and request.method == 'OPTIONS':
@@ -106,13 +113,16 @@ def cross_origin(origins=None, methods=None, headers=None,
             # Add a single Access-Control-Allow-Origin header, with either
             # the value of the Origin header or the string "*" as value.
             if wildcard:
-                # If the `origins` param is '*', either send the request's origin, or `*`
+                # If the `origins` param is '*', either send the request's
+                # origin, or `*`
                 if send_wildcard:
                     resp.headers[AccessControlAllowOrigin] = origins
                 else:
-                    resp.headers[AccessControlAllowOrigin] = request.headers.get('Origin', '*')
+                    req_origin = request.headers.get('Origin', '*')
+                    resp.headers[AccessControlAllowOrigin] = req_origin
 
-            # If not 'wildcard', send the string-joined-form of the origins header
+            # If not 'wildcard', send the string-joined-form of the
+            # origins header
             else:
                 resp.headers[AccessControlAllowOrigin] = origins_str
 
@@ -125,9 +135,17 @@ def cross_origin(origins=None, methods=None, headers=None,
                 resp.headers['Access-Control-Allow-Credentials'] = 'true'
             return resp
 
-        # Override Flask's default OPTIONS handling
-        f.required_methods = ['OPTIONS']
-        f.provide_automatic_options = automatic_options
+        # If True, intercept OPTIONS requests by modifying the view function
+        # mirroring Flask's default behavior, and wrapping the response with
+        # CORS headers.
+        #
+        # If f.provide_automatic_options is unset or True, Flask's route
+        # decorator (which is actually wraps the function object we return)
+        # intercepts OPTIONS handling, and requests will not have CORS headers
+        if automatic_options:
+            f.required_methods = getattr(f, 'required_methods', set())
+            f.required_methods.add('OPTIONS')
+            f.provide_automatic_options = False
 
         return update_wrapper(wrapped_function, f)
     return decorator
